/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * Decorator that disable the close methods. Instead of closing
 * is connection returned into related {@link DataSourceWrapper}.
 *
 * This class is used, created, managed by {@link DataSourceWrapper}
 * and serving only for internal purposes.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
class ConnectionWrapper extends ConnectionProxy
{
    /**
     * Reffering to datasource which is owner of this connection object.
     * The datasource is responsible for lifecycle of connection. Also closing
     * of connection is happening that connection is returned into datasource.
     */
    private final DataSourceWrapper dataSource;

    /**
     * flag that indicates whether connection is closed or not.
     * This flag is set to true if 'close()' method has been invoked.
     */
    private boolean isClosedFlag = false;


    /**
     * Constructor
     *
     * @param con
     * @param ds
     */
    public ConnectionWrapper(Connection con, DataSourceWrapper ds)
    {
        super(con);
        this.dataSource = ds;
    }


    /**
     * instead of closing the connection is returned into datasource
     * @throws SQLException
     */
    @Override
    public void close() throws SQLException
    {
        this.dataSource.releaseConnection(this);
        this.isClosedFlag = true;
        this.con = null;
    }


    /**
     * because there is no closing, this method is checked whether
     * connection is owned or not owned by datasource.
     */
    @Override
    public boolean isClosed() throws SQLException
    {
        return isClosedFlag;
    }
}
