/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * This datasource wrapper is used internally when you're using plain {@link Connection} instead of
 * datasource. It's because the {@link Sql} works with DataSources instead of plain connections.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class DataSourceWrapper extends AbstractDataSource {

    /**
     * Holds the connection you wish to use in datasource as
     * a single connection. Around this connection is created
     * {@link ConnectionWrapper proxy} that override close() methods.
     *
     * The connection is never closed inside datasource. Closing,
     * commit etc is managed via original connection.
     *
     * The getConnection method borrow the connection object and
     * then is this reference set to null because owner has been changed.
     *
     * The reference is back in moment when user close the {@link ConnectionWrapper proxy}
     * connection.
     *
     */
    private Connection wrappedConnection;


    /**
     * Constructor
     * @param wrappedConnection
     */
    public DataSourceWrapper(Connection wrappedConnection)
    {
        this.wrappedConnection = wrappedConnection;
    }


    /**
     * Method has no meaning for this kind of datasource because datasource
     * is not a connection creator. Datasource just holds the existing connection.
     */
    @Override
    public Connection getConnection(String username, String password) throws SQLException
    {
        throw new UnsupportedOperationException("this method is unsupported. Use getConnection() instead of this.");
    }


    @Override
    public Connection getConnection() throws SQLException
    {
        if (this.wrappedConnection != null) {
            ConnectionProxy proxy = new ConnectionWrapper(wrappedConnection, this);
            this.wrappedConnection = null;
            return proxy;
        }
        return null;
    }


    /**
     * releasing connection is process when connection is closed, that mean you
     * don't want to use it anymore. Instead of closing the connection is returned
     * to datasource which will own the connection until new getConnection() call
     * happens.
     *
     * @param proxy
     */
    void releaseConnection(ConnectionProxy proxy)
    {
        if (wrappedConnection == null) {
            this.wrappedConnection = proxy.getWrappedConnection();
        }
    }
}
