/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import java.sql.*;
import java.util.logging.Logger;

/**
 * Contains only static methods that simplifying some JDBC operations (like closing etc)
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class JDBCUtils {

    /** logger */
    private static Logger LOG = Logger.getLogger(JDBCUtils.class.getName());


    /**
     * Closing connection quietly
     */
    public static void closeQuietly(Connection connection)
    {
        if (connection != null) {
            try {
                connection.close();
            } catch (SQLException e) {
                LOG.warning("Error closing connection because: " + e.getMessage());
            }
        }
    }


    /**
     * Closing statement quietly
     */
    public static void closeQuietly(Statement stmt)
    {
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
                LOG.warning("Error closing statement because: " + e.getMessage());
            }
        }
    }


    /**
     * Closing prepared statement quietly
     */
    public static void closeQuietly(PreparedStatement pstmt)
    {
        if (pstmt != null) {
            try {
                pstmt.close();
            } catch (SQLException e) {
                LOG.warning("Error closing prepared statement because: " + e.getMessage());
            }
        }
    }


    /**
     * Closing result set quietly
     */
    public static void closeQuietly(ResultSet rs)
    {
        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException e) {
                LOG.warning("Error closing ResultSet because: " + e.getMessage());
            }
        }
    }


    /**
     * Closing connection statement and result set quietly in right order
     */
    public static void closeQuietly(Connection connection, Statement stmt, ResultSet rs)
    {
        closeQuietly(rs);
        closeQuietly(connection, stmt);
    }


    /**
     * Closing connection statement quietly in right order
     */
    public static void closeQuietly(Connection connection, Statement stmt)
    {
        closeQuietly(stmt);
        closeQuietly(connection);
    }


    /**
     * Closing connection statement and result set quietly in right order
     */
    public static void closeQuietly(Connection connection, PreparedStatement pstmt)
    {
        closeQuietly(pstmt);
        closeQuietly(connection);
    }


    /**
     * Closing connection statement quietly in right order
     */
    public static void closeQuietly(Connection connection, PreparedStatement pstmt, ResultSet rs)
    {
        closeQuietly(rs);
        closeQuietly(connection, pstmt);
    }


    /**
     * fill in values from incoming array into JDBC PreparedStatement.
     */
    public static void setValuesToPreparedStatement(Object[] values, PreparedStatement toStatemnet) throws SQLException
    {
        for (int i = 0; i < values.length; ++i) {
            Object val = values[i];
            int index = i + 1;

            if (val instanceof Long) {
                toStatemnet.setLong(index, (Long)val);
            } else if (val instanceof Integer) {
                toStatemnet.setInt(index, (Integer)val);
            } else if (val instanceof String) {
                toStatemnet.setString(index, (String)val);
            } else if (val instanceof Boolean) {
                toStatemnet.setBoolean(index, (Boolean)val);
            } else {
                toStatemnet.setObject(index, val);
            }
        }
    }


}
