/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import org.antlr.v4.runtime.*;

import java.io.*;
import java.nio.charset.Charset;

/**
 * The <b>Nyx</b> helps you with two topics. Simplify JDBC use and offer you
 * ability to execute whole SQL scripts. The Nyx doesn't manage transactions and connection
 * closing. Transaction management is still on you how you're using JDBC Connection.
 * <br>
 *
 * The main entry point is {@link com.celum.nyx.Sql} which is container of static methods.
 * You have to import the Sql as a static import:<br>
 *
 * <pre class="prettyprint">
 *     import static com.celum.nyx.Sql.*;
 * </pre>
 *
 *
 * <h3>Scripts and Queries</h3>
 * You might use Sql as a parser without any real DB or as a SQL executor
 * with real connection when SQLs are performed.<br>
 *
 * <br><i>example of SQL script parsing:</i>
 * <pre class="prettyprint">
 *    List&lt;String&gt; sqls = sql(new FileInputStream("script.sql")).parse().andReturnListOfQueries();
 * </pre>
 *
 * <br><i>example of SQL script execution:</i>
 * <pre class="prettyprint">
 *     sql(new FileInputStream("script.sql")).on(connection).run();
 * </pre>
 * <b>Note:</b> If something goes wrong during script execution, the exception is thrown and
 * Nyx terminate SQL execution. All other SQLs after broken SQL will not be executed.
 *
 * <br><i>example of SELECT query that returns one value:</i>
 * <pre class="prettyprint">
 *     int count = sql("SELECT COUNT(*) FROM some_table").on(connection).runQuery().andReturnInt();
 * </pre>
 *
 * <br><i>example of SELECT query that returns list of first column:</i>
 * <pre class="prettyprint">
 *     List&lt;String&gt; names = sql("SELECT name FROM some_table").on(connection).runQuery().andReturnListOfString();
 * </pre>
 *
 * <h3>Row Mapping</h3>
 * You might perform SQL selects and do row mapping which simplify SELECT's result
 * processing. You have to implement the {@link com.celum.nyx.mapper.RowMapper mapper} interface or
 * you could use one of the preddefined mappers in {@link com.celum.nyx.mapper} package.
 * <br>
 *
 * <br><i>example of SELECT and row {@link com.celum.nyx.mapper.RowMapper mapper}:</i>
 * <pre class="prettyprint">
 *     sql("SELECT * FROM some_table").on(connection).runQuery().mapRowsWith(
 *        new RowMapper() {
 *            public void map(ResultSet rs, long rownum) {
 *               String value = rs.getString(1);
 *               ...
 *            }
 *        }
 *     );
 * </pre>
 *
 *
 * <h3>Interceptors</h3>
 *
 * You could also modify each SQL command before comes into strategy via interceptors in
 * interceptor chain. You might have various interceptors like velocity interceptor which
 * replace placeholders in SQL by values etc. Interceptors could be used as with parser as
 * with SQL execution.<br>
 *
 * <br><i>example of interceptor usage:</i>
 * <pre class="prettyprint">
 *    sql(new FileInputStream("script.sql")).interceptWith(new VelocityInterceptor(values)).on(connection).run();
 * </pre>
 *
 *
 *
 * <h3>Binding variables</h3>
 *
 * Very often in commands like INSERT and UPDATE, the values are replaced by binding variables. As you know from JDBC,
 * they're  prepared statements for that purpose. Sql simplify also the prepared statements.<br>
 *
 * <br><i>example of simple insert:</i>
 * <pre class="prettyprint">
 *    sql("INSERT INTO some_table(name, email) VALUES (?, ?)")
 *        .on(connection)
 *        .runWithParams("mickeymouse", "mickeymouse@disneyland.com")
 *        .finish();
 * </pre>
 *
 * The JDBC prepared statements could be re-used with different values. It's same with Sql.<br>
 *
 * <br><i>example of multiple inserts:</i>
 * <pre class="prettyprint">
 *    sql("INSERT INTO some_table(name, email) VALUES (?, ?)")
 *        .on(connection)
 *        .runWithParams("mickeymouse", "mickeymouse@disneyland.com")
 *        .and("cinderella", "cinderella@disneyland.com")
 *        .and("bambi", "bambi@disneyland.com")
 *        .finish();
 * </pre>
 *
 * Sql SELECT queries could be parametrized as well.<br>
 *
 * <pre class="prettyprint">
 *     String filter = ...;
 *     int occurrence = sql("SELECT COUNT(*) FROM some_table WHERE column = ?")
 *                          .on(connection)
 *                          .runQueryWithParams(filter)
 *                          .andReturnInt();
 * </pre>
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class Sql {

    /**
     * Constructor is hidden because Sql is just a container
     * for all static methods
     */
    private Sql()
    {
    }


    public static SqlEngine sql(InputStream is)
    {
        try {
            return (new SqlEngine(new ANTLRInputStream(is)));
        } catch (IOException e) {
            throw new DataAccessException(e);
        }
    }


    public static SqlEngine sql(File f)
    {
        try {
            return (new SqlEngine(new ANTLRFileStream(f.getAbsolutePath())));
        } catch (IOException e) {
            throw new DataAccessException(e);
        }
    }


    public static SqlEngine sql(String sql)
    {
        return sql(new ByteArrayInputStream(sql.getBytes()));
    }


    public static PreparedStatementSetter batchOf(ValuesSetter... values)
    {
        return new BatchSetter(values);
    }


    public static ValuesSetter values(Object... values)
    {
        return new ValuesSetter(values);
    }

}
