/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import com.celum.nyx.interceptor.SqlInterceptor;
import org.antlr.v4.runtime.CharStream;
import org.antlr.v4.runtime.CommonTokenStream;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

/**
 * Engine is responsible for {@link com.celum.nyx.Sql Sql} fluent API and also for execution of SQL scripts.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class SqlEngine {

    /** carries the input stream of step */
    private final CharStream script;

    /** chain of interceptors which could affect the SQL command before it's executed */
    private final List<SqlInterceptor> interceptorChain = new LinkedList<SqlInterceptor>();


    /**
     * hidden constructor because class
     * is constructed via {@link com.celum.nyx.Sql Sql}
     */
    SqlEngine(CharStream script) {
        this.script = script;
    }


    public SqlEngine interceptWith(SqlInterceptor... interceptor)
    {
        for (SqlInterceptor i : interceptor) {
            if (i != null) {
                this.interceptorChain.add(i);
            }
        }
        return this;
    }


    public SqlOperations on(Connection con)
    {
        return new SqlOperations(new DataSourceWrapper(con));
    }


    public SqlOperations on(DataSource ds)
    {
        return new SqlOperations(ds);
    }


    public SqlParserOperations parse()
    {
        return new SqlParserOperations();
    }


    /**
     * part of {@link com.celum.nyx.Sql Sql} fluent chain
     */
    public class SqlOperations
    {
        private final DataSource ds;


        private SqlOperations(DataSource ds)
        {
            this.ds = ds;
        }

        public SqlResultSet runQuery()
        {
            return SqlEngine.this.run(new StatementQueryCallback(this.ds)).andGetResultSet();
        }


        public SqlResultSet runQueryWith(ValuesSetter values)
        {
            return SqlEngine.this.run(new PreparedStatementQueryCallback(ds, values)).andGetResultSet();
        }


        public void run()
        {
            SqlEngine.this.run(new StatementCallback(ds));
        }


        public void runWith(PreparedStatementSetter values)
        {
            SqlEngine.this.run(new PreparedStatementUpdateCallback(ds, values));
        }
    }

    /**
     * part of {@link com.celum.nyx.Sql Sql} fluent chain
     */
    public class SqlParserOperations
    {
        public List<String> andReturnListOfQueries()
        {
            return withSqlCallback(new ListParser()).andReturnListOfQueries();
        }

        public <R extends SqlCallback> R withSqlCallback(R sqlCallback)
        {
            return SqlEngine.this.run(sqlCallback);
        }
    }


    /**
     * Method load and parse SQL step into sql queries and call
     * {@link SqlCallback callback} 'doSql()' method for each query.
     *
     * @param callback determines how the Query will be processed
     *
     * @return method return the same callback you fill in because DSL reasons
     *         and callback could contain the results of loading.
     */
    private <R extends SqlCallback> R run(R callback)
    {
        //create and prepare lexer, parser and grammar-listener which store the result
        SqlGrammarLexer lexer = new SqlGrammarLexer(script);
        SqlGrammarParser parser = new SqlGrammarParser(new CommonTokenStream(lexer));
        ParserGrammarListener pgl = new ParserGrammarListener(callback, interceptorChain);
        parser.addParseListener(pgl);

        //parsing the SQL step
        parser.sql();
        return callback;
    }

}
