/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import com.celum.nyx.mapper.*;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * part of {@link com.celum.nyx.Sql Sql} fluent chain that offer you
 * data processing after SQL execution.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class SqlResultSet
{
    private final SqlCloseable whatToClose;

    /** when query is executed, then is filled by result set */
    private final ResultSet rs;

    /**
     * Constructor
     * @param whatToClose
     * @param rs
     */
    SqlResultSet(SqlCloseable whatToClose, ResultSet rs)
    {
        this.whatToClose = whatToClose;
        this.rs = rs;
    }


    /**
     * Goes through result set and call {@link com.celum.nyx.mapper.RowMapper} map() for
     * each row.
     */
    public <T extends RowMapper> T mapRowsWith(T rowMapper)
    {
        try {
            long rownum = 0;
            while (rs.next()) {
                rowMapper.map(rs,rownum);
                rownum++;
            }
        } catch (SQLException e) {
            throw new DataAccessException("SQL error:" + e.getMessage(), e);
        } finally {
            whatToClose.close();
        }

        return rowMapper;
    }


    /**
     * returns string value from first columnd from first row.
     * If select returns empty rows, the 'null' value is returned.
     */
    public String andReturnString()
    {
        return andReturnString(null);
    }


    /**
     * Return string value in first row, in first column. If
     * there is no row or column the defaultValue is returned.
     */
    public String andReturnString(String defaultValue)
    {
        try {
            if (rs.next()) {
                return rs.getString(1);
            } else {
                return defaultValue;
            }
        } catch (SQLException e) {
            throw new DataAccessException("SQL error:" + e.getMessage(), e);
        } finally {
            whatToClose.close();
        }
    }


    /**
     * returns long value from first row in first column. If
     * the select returns empty rows, the '0' is returned.
     */
    public long andReturnLong()
    {
        return andReturnLong(0);
    }


    /**
     * returns long value in first row, in first column. It
     * there is no row or column, the defaultValue is returned.
     */
    public long andReturnLong(long defaultValue)
    {
        try {
            if (rs.next()) {
                return rs.getLong(1);
            } else {
                return defaultValue;
            }
        } catch (SQLException e) {
            throw new DataAccessException("SQL error:" + e.getMessage(), e);
        } finally {
            whatToClose.close();
        }
    }


    /**
     * returns int value in first row and first column. If there
     * is no value present, the 0 is returned
     */
    public int andReturnInt()
    {
        return andReturnInt(0);
    }


    /**
     * returns int value in first row, in first column. If there is no
     * row, the defaultValue is returned.
     */
    public int andReturnInt(int defaultValue)
    {
        try {
            if (rs.next()) {
                return rs.getInt(1);
            } else {
                return defaultValue;
            }
        } catch (SQLException e) {
            throw new DataAccessException("SQL error:" + e.getMessage(), e);
        } finally {
            whatToClose.close();
        }
    }


    /**
     * returns boolean value from first row in first column. If
     * the select returns empty rows, the 'false' is returned.
     */
    public boolean andReturnBoolean()
    {
        return andReturnBoolean(false);
    }


    /**
     * returns boolean value in first row, in first column. It
     * there is no row or column, the defaultValue is returned
     */
    public boolean andReturnBoolean(boolean defaultValue)
    {
        try {
            if (rs.next()) {
                return rs.getBoolean(1);
            } else {
                return defaultValue;
            }
        } catch (SQLException e) {
            throw new DataAccessException("SQL error:" + e.getMessage(), e);
        } finally {
            whatToClose.close();
        }
    }


    /**
     * returns list of boolean values taken from first column.
     */
    public List<Boolean> andReturnListOfBoolean()
    {
        return mapRowsWith(new BooleanListMapper()).andReturnList();
    }


    /**
     * returns list of string values taken from first column.
     */
    public List<String> andReturnListOfString()
    {
        return mapRowsWith(new StringListMapper()).andReturnList();
    }


    /**
     * returns list of integer values taken from first column.
     */
    public List<Integer> andReturnListOfInteger()
    {
        return mapRowsWith(new IntegerListMapper()).andReturnList();
    }


    /**
     * returns list of long values taken from first column.
     */
    public List<Long> andReturnListOfLong()
    {
        return mapRowsWith(new LongListMapper()).andReturnList();
    }
}
